/*
 * rc-deps-util.c
 *
 * Copyright (C) 2000, 2001   Ximian, Inc.
 *
 * Authors: Vladimir Vukicevic <vladimir@ximian.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <string.h>

#include "rc-package-dep.h"
#include "rc-debman-general.h"

#include "rc-deps-util.h"

RCDepsFixInfo *
rc_deps_fix_system (RCPackageSList *system_packages,
                    RCChannelSList *subscribed_channels,
                    RCChannelSList *unsubscribed_channels)
{
    RCDepsFixInfo *fixinfo;
    GSList *dupes, *dupes_iter;
    RCPackageSList *munged_system_packages;
    RCPackageSList *removed_dupes = NULL;
    RCDepResolution *rcdr;

    fixinfo = g_new0 (RCDepsFixInfo, 1);

    dupes = rc_package_slist_find_duplicates (system_packages);

    munged_system_packages = g_slist_copy (system_packages);

    /* We're going to do something dangerous here and simply remove all
     * but the newest version of all duplicate system packages.
     * Note that if something depends on the older version, it will get caught
     * in the dependency check.  So technically, if all the dupes are removed,
     * and the dep results are followed, everything will be okay....
     */

    dupes_iter = dupes;
    while (dupes_iter) {
        RCPackageSList *pkg_dupe = (RCPackageSList *) dupes_iter->data;
        RCPackageSList *dup_iter;

        pkg_dupe = rc_package_slist_sort_by_spec (pkg_dupe);

        dup_iter = pkg_dupe;
        do {
            RCPackage *dupe = (RCPackage *) dup_iter->data;
            removed_dupes = g_slist_append (removed_dupes, dupe);
            munged_system_packages = g_slist_remove (munged_system_packages, dupe);
            dup_iter = dup_iter->next;
        } while (dup_iter->next);

        g_slist_free (pkg_dupe);
        dupes_iter = dupes_iter->next;
    }

    g_slist_free (dupes);

    rcdr = rc_dep_resolution_new (munged_system_packages,
                                  NULL,
                                  subscribed_channels,
                                  unsubscribed_channels,
                                  0);

    fixinfo->resolution_result = rc_dep_resolution_get_result (rcdr);
    fixinfo->duplicate_packages = removed_dupes;

    /* Evil */
    fixinfo->resolution_result->packages_to_remove =
        g_slist_concat (fixinfo->resolution_result->packages_to_remove,
                        removed_dupes);

    return fixinfo;
}


#if 0

/*** This code is a work in progress and as such is not used yet. ***/

/*
libesd0 (>= 0.2.20) | libesd-alsa0 (>= 0.2.20)
imlib1 | gdk-imlib1
ddd | xxgdb
perl5 | perl (>= 5.003)
exim | smail | sendmail | mail-transport-agent


(libesd0&>=&0.2.20|libesd-alsa0&>=&0.2.20)
*/

gchar *
munge_or_dep_name (RCPackageDep *dep)
{
    GString *gstr;
    char *out_str;

    gstr = g_string_sized_new (50);

    g_string_append (gstr, "(||");

    while (dep) {
        RCPackageDepItem *pdi = (RCPackageDepItem *) dep->data;

        if (pdi->relation & RC_RELATION_WEAK) {
            g_error ("Bork bork bork! munge or dep with weak dep!");
        }

        g_string_append (gstr, pdi->spec.name);

        if (pdi->relation != RC_RELATION_ANY) {
            const gchar *rel = rc_package_relation_to_string (pdi->relation, FALSE);
            g_string_append (gstr, "&");
            g_string_append (gstr, rel);
            g_string_append (gstr, "&");

            if (pdi->spec.epoch) {
                gchar *s = g_strdup_printf ("%d:", pdi->spec.epoch);
                g_string_append (gstr, s);
                g_free (s);
            }

            g_string_append (gstr, pdi->spec.version);

            if (pdi->spec.release) {
                g_string_append (gstr, "-");
                g_string_append (gstr, pdi->spec.release);
            }
        }

        if (dep->next) {
            g_string_append (gstr, "|");
        }
        dep = dep->next;
    }

    g_string_append (gstr, ")");

    out_str = g_strdup (gstr->str);
    g_string_free (gstr, TRUE);

    return out_str;
}

RCPackageDep *
split_munged_or_dep_string (gchar *munged)
{
    gchar *s, *p, *zz;
    RCPackageDep *out_dep = NULL;
    gboolean have_more = TRUE;

    s = munged;
    if (strncmp (s, "(||", 3)) {
        g_warning ("'%s' is not a munged or string!\n");
        return NULL;
    }

    s += 3;

    zz = strchr (s, ')');

    /* s now points to the start of the first thing */
    do {
        RCPackageDepItem *cur_item;
        char *z;
        gchar *name;

        /* grab the name */

        cur_item = g_new0 (RCPackageDepItem, 1);

        z = strchr (s, '|');
        p = strchr (s, '&');

        if (!z) {
            have_more = FALSE;
        }

        name = g_strndup (s, p ? p - s : (z ? z - s : zz - s));
        cur_item->spec.name = name;

        if (p) {
            char *e;
            char op[4];
            char *vstr;

            /* We need to parse version things */
            p++;
            e = strchr (p, '&');
            if (!e) {
                /* Bad. */
                g_error ("Couldn't parse ver str");
            }

            /* text between p and e is an operator */
            strncpy (op, p, e - p);
            op[e - p] = 0;
            cur_item->relation = rc_string_to_package_relation (op);

            e++;
            if (z) {
                p = z;
            } else {
                p = zz;
            }

            /* e .. p is the epoch:version-release */
            vstr = g_strndup (e, p - e);
            rc_debman_parse_version (vstr, &cur_item->spec.epoch,
                                     &cur_item->spec.version,
                                     &cur_item->spec.release);
            g_free (vstr);
        }

        out_dep = g_slist_append (out_dep, cur_item);
        s = z + 1;

        if (p == zz)
            have_more = FALSE;
    } while (have_more);

    return out_dep;
}

/**** end of not used code ***/

#endif
