/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-progress.c: Progress window
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include <gui.h>

#include "gui-download.h"

gboolean inverted = FALSE;
GUIInstallType install_type;

char *progress_icons[] = {
    "progress-download.png",
    "progress-verify.png",
    "progress-config.png",
    "progress-install.png",
    "progress-finished.png",
    "progress-finished.png"
};

#define H "#ffffff"
#define A "#000000"
#define P "#a3a3a3"

static char *
html_color(GUIInstallProgress colorize, GUIInstallProgress current, 
           gboolean hide)
{
    char *ACTIVE = A;
    char *PASSED = P;
    char *HIDDEN = hide ? H : P;

    char *color_table[6][4] = {
                                     /* D */ /* V */ /* C */ /* I */
        /* PROGRESS_DOWNLOAD */    { ACTIVE, HIDDEN, HIDDEN, HIDDEN },
        /* PROGRESS_VERIFY   */    { PASSED, ACTIVE, HIDDEN, HIDDEN },
        /* PROGRESS_CONFIG   */    { PASSED, PASSED, ACTIVE, HIDDEN },
        /* PROGRESS_INSTALL  */    { PASSED, PASSED, PASSED, ACTIVE },
        /* PROGRESS_WAIT_FOR_IT */ { PASSED, PASSED, PASSED, PASSED },
        /* PROGRESS_FINISHED */    { PASSED, PASSED, PASSED, PASSED }
    };

    if (inverted) {
        color_table[PROGRESS_CONFIG][PROGRESS_INSTALL] = PASSED;
        color_table[PROGRESS_INSTALL][PROGRESS_CONFIG] = HIDDEN;
    }

    return color_table[current][colorize];
} /* html_color */

static void
gtk_progress_sucks_my_ass(GtkProgress *progress)
{
    GtkWidget *widget;

    g_return_if_fail (progress != NULL);
    g_return_if_fail (GTK_IS_PROGRESS (progress));

    if (progress->offscreen_pixmap)
        gdk_pixmap_unref(progress->offscreen_pixmap);

    if (GTK_WIDGET_REALIZED(progress)) {
        widget = GTK_WIDGET(progress);

        progress->offscreen_pixmap = gdk_pixmap_new(
            widget->window, widget->allocation.width,
            widget->allocation.height, -1);
        GTK_PROGRESS_CLASS(GTK_OBJECT(progress)->klass)->paint(progress);
    }
    else
        progress->offscreen_pixmap = NULL;
}

static void
rc_gui_set_progress_install(GUIInstallProgress step, int total, int done, 
char *name)
{
    GtkHTMLStream *stream;
    char *h, *h1;
    GtkWidget *single, *tot;

    static int download_done;
    static int verify_done;
    static int install_done;
    static int config_done;

    static int download_total;
    static int verify_total;
    static int install_total;
    static int config_total;

    single = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "single_progress");
    if (single->parent) {
        gtk_widget_ref(single);
        gtk_container_remove(GTK_CONTAINER(single->parent), single);
    }

    tot = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "total_progress");
    if (tot->parent) {
        gtk_widget_ref(tot);
        gtk_container_remove(GTK_CONTAINER(tot->parent), tot);
    }

    switch (step) {
    case PROGRESS_DOWNLOAD:
        download_done = done;
        download_total = total;
        /* Reset the others */
        verify_done = 0;
        install_done = 0;
        config_done = 0;
        verify_total = 0;
        install_total = 0;
        config_total = 0;
        break;
    case PROGRESS_VERIFY:
        verify_done = done;
        verify_total = total;
        break;
    case PROGRESS_INSTALL:
        install_done = done;
        install_total = total;
        break;
    case PROGRESS_CONFIG:
        config_done = done;
        config_total = total;
        break;
    case PROGRESS_WAIT_FOR_IT:
    case PROGRESS_FINISHED:
        break;
    default:
        g_assert_not_reached();
    }

    stream = gtk_html_begin(appinfo->download_html);
    h = g_strdup_printf(
        "<html><body marginwidth=0 marginheight=0 bgcolor=#e5e3e3>"
        "<table width=100%% cellspacing=0 cellpadding=2 bgcolor=#ffffff>"
        "<tr valign=top>"
        "<td rowspan=4 width=100>"
        "<img src=\"file://"SHAREDIR"/%s\" width=100 height=100>"
        "</td>"
        "<td width=120>"
        "<font size=\"+2\" color=%s><b>Downloading</b></font></td>"
        "<td><font color=%s>"
        "First, the requested packages are downloaded from their "
        "source.</font>"
        "<font color=%s>"
        "<table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>Done:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;Remaining:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;Percent Complete:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table>"
        "</font>"
        "</td>"
        "<td align=right width=66>"
        "<a href=\"updater:download_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30 "
        "     border=no></a></td>"
        "</tr>"
        "<tr valign=top>"
        "<td width=100>"
        "<font size=\"+2\" color=%s><b>Verifying</b></font></td>"
        "<td><font color=%s>"
        "Next, packages will be verified to ensure cryptographic "
        "integrity.</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>Done:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;Remaining:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;Percent Complete:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:verify_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30 "
        "     border=no></a></td>"
        "</tr>",
        progress_icons[step],
        html_color(PROGRESS_DOWNLOAD, step, FALSE),
        html_color(PROGRESS_DOWNLOAD, step, FALSE),
        html_color(PROGRESS_DOWNLOAD, step, TRUE),
        download_done, (download_total - download_done),
        (int) ((double) download_done / (double) download_total * 100.0),
        html_color(PROGRESS_VERIFY, step, FALSE),
        html_color(PROGRESS_VERIFY, step, FALSE),
        html_color(PROGRESS_VERIFY, step, TRUE),
        verify_done, (verify_total - verify_done),
        (int) ((double) verify_done / (double) verify_total * 100.0));
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);
    h = g_strdup_printf(
        "<tr valign=top>"
        "<td width=100>"
        "<font size=\"+2\" color=%s><b>Configuring</b></font></td>"
        "<td><font color=%s>%s</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>Done:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;Remaining:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;Percent Complete:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:config_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30"
        "     border=no></a></td>"
        "</tr>",
        html_color(PROGRESS_CONFIG, step, FALSE),
        html_color(PROGRESS_CONFIG, step, FALSE),
        inverted ? "Finally, the installed packages are configured" :
        "Preparing to install downloaded packages",
        html_color(PROGRESS_CONFIG, step, TRUE),
        config_done, (config_total - config_done),
        (int) ((double) config_done / (double) config_total * 100.0));
    h1 = g_strdup_printf(
        "<tr valign=top>"
        "<td width=100>"
        "<font size=\"+2\" color=%s><b>Installing</b></font></td>"
        "<td><font color=%s>"
        "%she verified packages are installed on your system.</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>Done:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;Remaining:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;Percent Complete:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:install_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30"
        "     border=no></a></td>"
        "</tr>",
        html_color(PROGRESS_INSTALL, step, FALSE),
        html_color(PROGRESS_INSTALL, step, FALSE),
        inverted ? "T" : "Finally, t",
        html_color(PROGRESS_INSTALL, step, TRUE),
        install_done, (install_total - install_done),
        (int) ((double) install_done / (double) install_total * 100.0));
    if (inverted) {
        gtk_html_write(appinfo->download_html, stream, h1, strlen(h1));
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    }
    else {
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
        gtk_html_write(appinfo->download_html, stream, h1, strlen(h1));
    }
    g_free(h);
    g_free(h1);

    if (step == PROGRESS_WAIT_FOR_IT)
        h1 = "Finishing up...";
    else if (step == PROGRESS_FINISHED)
        h1 = "Installation has finished.";
    else {
        h1 = 
            "The packages you requested are being downloaded and installed "
            "on your system.";
    }

    h = g_strdup_printf(
        "</table>"
        "<table width=100%% bgcolor=#000000 cellspacing=0 cellpadding=0>"
        "<tr><td height=4>&nbsp;</td></tr>"
        "</table>"
        "<table width=100%% bgcolor=#e5e3e3 cellspacing=0 cellpadding=4"
        "       height=100%%>"
        "<tr>"
        "<td align=center colspan=2>%s</td>"
        "</tr>", h1);
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step == PROGRESS_DOWNLOAD) {
        h = g_strdup_printf(
            "<tr><td><b>%s</b></td></tr>"
            "<tr><td><object classid=\"single_progress\"></object></td></tr>",
            name ? name : "");
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
        g_free(h);
    }

    if (step != PROGRESS_WAIT_FOR_IT && step != PROGRESS_FINISHED) {
        h = "<tr><td><b>Total</b></td></tr>"
            "<tr><td><object classid=\"total_progress\"></object></td></tr>";
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    }

    h = "</table></body></html>";
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    gtk_html_end(appinfo->download_html, stream, GTK_HTML_STREAM_OK);

    if (step == PROGRESS_WAIT_FOR_IT) {
        gtk_progress_sucks_my_ass(GTK_PROGRESS(single));
        gtk_progress_sucks_my_ass(GTK_PROGRESS(tot));
    }

    if (step != PROGRESS_DOWNLOAD) {
        gtk_progress_configure(
            GTK_PROGRESS(tot), (gdouble) done, 0.0, (gdouble) total);
    }
} /* rc_gui_set_progress_install */

static void
rc_gui_set_progress_remove(GUIInstallProgress step, int total, int done, 
                           char *name)
{
    GtkHTMLStream *stream;
    char *h, *h1;
    GtkWidget *single, *tot;

    static int remove_done;

    single = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "single_progress");
    if (single->parent) {
        gtk_widget_ref(single);
        gtk_container_remove(GTK_CONTAINER(single->parent), single);
    }

    tot = gtk_object_get_data(
        GTK_OBJECT(appinfo->download_html), "total_progress");
    if (tot->parent) {
        gtk_widget_ref(tot);
        gtk_container_remove(GTK_CONTAINER(tot->parent), tot);
    }

    switch (step) {
    case PROGRESS_INSTALL:
        remove_done = done;
        break;
    case PROGRESS_WAIT_FOR_IT:
    case PROGRESS_FINISHED:
        break;
    default:
        g_assert_not_reached();
    }

    stream = gtk_html_begin(appinfo->download_html);
    h = g_strdup_printf(
        "<html><body marginwidth=0 marginheight=0 bgcolor=#e5e3e3>"
        "<table width=100%% cellspacing=0 cellpadding=2 bgcolor=#ffffff>"
        "<tr valign=center>" /* valign=top */
        "<td rowspan=4 width=100>"
        "<img src=\"file://"SHAREDIR"/%s\" width=100 height=100>"
        "</td>"
        "<td width=100>"
        "<font size=\"+2\" color=%s><b>Removing</b></font></td>"
        "<td><font color=%s>"
        "The packages you selected are being removed from your system.</font>"
        "<font color=%s><table cellspacing=0 cellpadding=0><tr>"
        "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>"
        "<td width=40>Done:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=85>&nbsp;&nbsp;&nbsp;&nbsp;Remaining:</td>"
        "<td width=30><b>%d</b></td>"
        "<td width=130>&nbsp;&nbsp;&nbsp;&nbsp;Percent Complete:</td>"
        "<td width=30><b>%d%%</b></td>"
        "</tr></table></font></td>"
        "<td align=right width=66>"
        "<a href=\"updater:remove_log\">"
        "<img src=\"file://"SHAREDIR"/info-off.png\" width=66 height=30"
        "     border=no></a></td>"
        "</tr>",
        progress_icons[step],
        html_color(PROGRESS_INSTALL, step, FALSE),
        html_color(PROGRESS_INSTALL, step, FALSE),
        html_color(PROGRESS_INSTALL, step, TRUE),
        remove_done, (total - remove_done),
        (int) ((double) remove_done / (double) total * 100.0));
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step == PROGRESS_WAIT_FOR_IT)
        h1 = "Finishing up...";
    else if (step == PROGRESS_FINISHED)
        h1 = "Removal has finished.";
    else
        h1 = "The packages you selected are being removed from you system.";

    h = g_strdup_printf(
        "</table>"
        "<table width=100%% bgcolor=#000000 cellspacing=0 cellpadding=0>"
        "<tr><td height=4>&nbsp;</td></tr>"
        "</table>"
        "<table width=100%% bgcolor=#e5e3e3 cellspacing=0 cellpadding=4"
        "       height=100%%>"
        "<tr>"
        "<td align=center colspan=2>%s</td>"
        "</tr>", h1);
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    g_free(h);

    if (step != PROGRESS_WAIT_FOR_IT && step != PROGRESS_FINISHED) {
        h = 
            "<tr><td><b>Total</b></td></tr>"
            "<tr><td><object classid=\"total_progress\"></object></td></tr>";
        gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    }

    h = "</table></body></html>";
    gtk_html_write(appinfo->download_html, stream, h, strlen(h));
    gtk_html_end(appinfo->download_html, stream, GTK_HTML_STREAM_OK);

    if (step == PROGRESS_WAIT_FOR_IT) {
        gtk_progress_sucks_my_ass(GTK_PROGRESS(single));
        gtk_progress_sucks_my_ass(GTK_PROGRESS(tot));
    }

    gtk_progress_configure(
        GTK_PROGRESS(tot), (gdouble) done, 0.0, (gdouble) total);
} /* rc_gui_set_progress_remove */

void
rc_gui_set_progress(GUIInstallProgress step, int total, int done, char *name)
{
    if (install_type == GUI_INSTALL)
        rc_gui_set_progress_install(step, total, done, name);
    else if (install_type == GUI_REMOVE)
        rc_gui_set_progress_remove(step, total, done, name);
    else
        g_assert_not_reached();
} /* rc_gui_set_progress */

void
rc_gui_set_progress_inverted(gboolean inv)
{
    inverted = inv;
} /* rc_gui_set_progress_inverted */

GUIInstallType
rc_gui_get_install_type(void)
{
    return install_type;
} /* rc_gui_get_install_type */

void
rc_gui_prepare_install_page(GUIInstallType type)
{
    GtkHTMLStream *stream;
    char *h;

    install_type = type;

    stream = gtk_html_begin(GTK_HTML(GET_WIDGET("download_head_html")));
    h = g_strdup_printf(
        "<html><body bgcolor=#000000 text=#ffffff marginwidth=0"
        "            marginheight=0>"
        "<table cellspacing=0 cellpadding=1 width=100%% border=0>"
        "<tr><td align=left>"
        "<img src=\"file://"SHAREDIR"/download.png\">"
        "</td><td align=left width=100%%>"
        "<font size=\"+2\"><b>Package %s</b></font>"
        "</td></tr></table></body></html>",
        type == GUI_INSTALL ? "Installation" : "Removal");
    gtk_html_write(GTK_HTML(GET_WIDGET("download_head_html")), stream, h,
                   strlen(h));
    gtk_html_end(GTK_HTML(GET_WIDGET("download_head_html")), stream,
                 GTK_HTML_STREAM_OK);
} /* rc_gui_prepare_install_page */

static void
size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
{
    gtk_widget_set_usize (GTK_WIDGET (data), allocation->width - 8, -1);
}

void
rc_gui_init_install_page(void)
{
    GtkWidget *w;

    w = gtk_progress_bar_new();
    gtk_progress_set_show_text(GTK_PROGRESS(w), TRUE);
    gtk_widget_show(w);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->download_html), "single_progress", w);
    gtk_signal_connect (
        GTK_OBJECT (appinfo->download_html), "size-allocate",
        GTK_SIGNAL_FUNC (size_allocate), (gpointer) w);

    w = gtk_progress_bar_new();
    gtk_progress_set_show_text(GTK_PROGRESS(w), TRUE);
    gtk_widget_show(w);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->download_html), "total_progress", w);
    gtk_signal_connect (
        GTK_OBJECT (appinfo->download_html), "size-allocate",
        GTK_SIGNAL_FUNC (size_allocate), (gpointer) w);
} /* rc_gui_init_install_page */
