/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui-glade.c: Callbacks for widgets defined in Glade
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <fcntl.h>
#include <crypt.h>
#include <time.h>

#include <gnome.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include <libgnomeui/gnome-window-icon.h>

#include <gtkhtml/htmlimage.h>
#include <gtkhtml/htmlengine-search.h>

#include <gal/e-paned/e-paned.h>
#include <gal/e-paned/e-hpaned.h>
#include <gal/widgets/e-scroll-frame.h>

#include "gui.h"
#include "gui-prefs.h"
#include "cache.h"
#include "gui-install.h"
#include "mirror.h"
#include "subscription.h"

#include "libredcarpet.h"

#include "gui-glade.h"

#include "gui-uri.h"
#include "gui-util.h"
#include "gui-html.h"

static void on_set_base (GtkHTML *html, const gchar *url, gpointer data);

/* Glade callbacks */
void search_clicked(GtkWidget *item);
void search_close_clicked(GtkWidget *button);
void search_case_toggled(GtkWidget *button);
void search_button_clicked(GtkWidget *button);
void dont_show_clicked(GtkToggleButton *button);
void select_all_install(GtkWidget *item);
void select_none_install(GtkWidget *item);
void select_all_update(GtkWidget *item);
void select_none_update(GtkWidget *item);
void on_install_local_file_activate(GtkWidget *item);
void on_verify_installed_packages_activate(GtkWidget *item);

GtkWidget *
rc_scroll_html_widget_new (gchar *widget_name, gchar *s1, gchar *s2, int scroll, int shadow)
{
    GtkWidget *html, *frame;

    gtk_widget_push_visual(gdk_rgb_get_visual());
    gtk_widget_push_colormap(gdk_rgb_get_cmap());

    html = gtk_html_new();
#if 0
    gtk_html_allow_selection (GTK_HTML (html), FALSE);
#endif

    gtk_widget_pop_colormap();
    gtk_widget_pop_visual();

    gtk_signal_connect(
        GTK_OBJECT(html), "url_requested", 
        GTK_SIGNAL_FUNC(rc_gui_html_load_image), NULL);
    gtk_signal_connect(
        GTK_OBJECT(html), "object_requested",
        GTK_SIGNAL_FUNC(rc_gui_html_object_requested), NULL);
    gtk_signal_connect(
        GTK_OBJECT(html), "set_base",
        GTK_SIGNAL_FUNC(on_set_base), NULL);
    if (scroll) {
        frame = e_scroll_frame_new(NULL, NULL);
        
        e_scroll_frame_set_policy(E_SCROLL_FRAME(frame),
                                  scroll 
                                  ? GTK_POLICY_AUTOMATIC
                                  : GTK_POLICY_NEVER,
                                  scroll
                                  ? GTK_POLICY_AUTOMATIC
                                  : GTK_POLICY_NEVER);

#if 0        
        e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (frame),
                                        shadow 
                                        ? GTK_SHADOW_IN
                                        : GTK_SHADOW_NONE);
        gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
#else
        e_scroll_frame_set_shadow_type(E_SCROLL_FRAME(frame), GTK_SHADOW_NONE);
        gtk_container_set_border_width (GTK_CONTAINER (frame), 
                                        shadow ? 2 : 0);
#endif
        
        gtk_container_add(GTK_CONTAINER (frame), html);
    } else
        frame = html;

    gtk_widget_show_all(frame);

    gtk_object_set_user_data(GTK_OBJECT (frame), html);
    return frame;        
} /* rc_gui_scroll_html_widget_new */

GtkWidget *
rc_main_e_paned_new (gchar *widget_name, gchar *s1, gchar *s2,
                     gint i1, gint i2)
{
    GtkWidget *paned;

    gtk_widget_push_visual (gdk_rgb_get_visual ());
    gtk_widget_push_colormap (gdk_rgb_get_cmap ());

    paned = e_hpaned_new ();

    gtk_widget_pop_colormap ();
    gtk_widget_pop_visual ();

    gtk_widget_show (paned);

    return (paned);
}

void
exit_clicked(void)
{
    gtk_exit (0);
} /* exit_clicked */

void
search_clicked(GtkWidget *item)
{
    gtk_widget_show(GET_WIDGET("search_dialog"));
    gtk_widget_grab_focus(GET_WIDGET("search_entry"));
} /* search_clicked */

void
search_close_clicked(GtkWidget *button)
{
    gtk_widget_hide(GET_WIDGET("search_dialog"));
} /* search_close_clicked */

void
search_case_toggled(GtkWidget *button)
{
    gpointer s;

    appinfo->case_sensitive = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(button));
    /* We're resetting the search parameters because GtkHTML doesn't allow you
       to change them on the fly. Bummer. */
    s = gtk_object_get_data(GTK_OBJECT(appinfo->update_html), "last_search");
    g_free(s);
    gtk_object_set_data(GTK_OBJECT(appinfo->update_html), "last_search", NULL);

    s = gtk_object_get_data(
        GTK_OBJECT(appinfo->available_html), "last_search");
    g_free(s);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->available_html), "last_search", NULL);

    s = gtk_object_get_data(
        GTK_OBJECT(appinfo->installed_html), "last_search");
    g_free(s);
    gtk_object_set_data(
        GTK_OBJECT(appinfo->installed_html), "last_search", NULL);
} /* search_case_toggled */

void
rc_gui_do_search(char *text)
{
    MainNotebookPage spage;
    MedianNotebookPage mpage;
    MinorNotebookPage page;
    GtkHTML *html = NULL;
    gboolean ok = FALSE;
    char *last_search;
    int (*cmp_func)(const char *s1, const char *s2);

    if (!strcmp (text, "ALL YOUR BASE ARE BELONG TO US")) {
        GtkWidget *w;
        w = gnome_ok_dialog ("SOMEBODY SET UP US THE BOMB\nMAIN SCREEN TURN ON!\nRED CARPET: FOR GREAT JUSTICE.");
        gnome_dialog_run_and_close (GNOME_DIALOG (w));
//        rc_gui_handle_uri (NULL, "updater:web?http://www.intrepid.com/~vladimir/ayb.html", NULL);
        return;
    }

    spage = gtk_notebook_get_current_page(
        GTK_NOTEBOOK(GET_WIDGET("main_notebook")));

    switch (spage) {
    case PAGE_VERIFY:
        html = GTK_HTML(gtk_object_get_user_data(
            GTK_OBJECT(GET_WIDGET("verify_html"))));
        break;
    case PAGE_ERROR:
        html = GTK_HTML(gtk_object_get_user_data(
            GTK_OBJECT(GET_WIDGET("error_html"))));
        break;
    case PAGE_REGISTER:
        html = GTK_HTML(gtk_object_get_user_data(
            GTK_OBJECT(GET_WIDGET("register_html"))));
        break;
    case PAGE_DOWNLOAD:
        rc_debug (RC_DEBUG_LEVEL_WARNING, "Searching not allowed here\n");
        return;
        break;
    case PAGE_CHANNELS:
        mpage = gtk_notebook_get_current_page(
            GTK_NOTEBOOK(GET_WIDGET("minor_notebook")));
        if (mpage == PAGE_SUMMARY)
            html = appinfo->exec_body;
        else {
            page = gtk_notebook_get_current_page(
                GTK_NOTEBOOK(GET_WIDGET("notebook")));
            switch (page) {
            case PAGE_UPDATES:
                html = appinfo->update_html;
                break;
            case PAGE_INSTALLED:
                html = appinfo->installed_html;
                break;
            case PAGE_AVAILABLE:
                html = appinfo->available_html;
                break;
            case PAGE_INFO:
                html = appinfo->info_html;
                break;
            case PAGE_PREFERENCES:
                rc_debug (RC_DEBUG_LEVEL_WARNING, "Searching not allowed here\n");
                return;
                break;
            default:
                g_assert_not_reached();
            }
        }
        break;
    default:
        g_assert_not_reached();
    }

    if (appinfo->case_sensitive)
        cmp_func = strcmp;
    else
        cmp_func = g_strcasecmp;
    
    g_return_if_fail(html->engine);

    last_search = gtk_object_get_data(GTK_OBJECT(html), "last_search");
    if (last_search && (*cmp_func)(text, last_search) == 0) {
        ok = html_engine_search_next(html->engine);
    }

    if (!ok) {
        ok = html_engine_search(
            html->engine, text, appinfo->case_sensitive, TRUE, FALSE);
        g_free(last_search);
        gtk_object_set_data(GTK_OBJECT(html), "last_search", g_strdup(text));
    }

    if (!ok) {
        /* Wrap-around */
        ok = html_engine_search_next(html->engine);
    }

    if (!ok) {
        GtkWidget *err;

        err = gnome_error_dialog(_("Keyword not found"));
        gnome_dialog_run_and_close(GNOME_DIALOG(err));
    }
} /* rc_gui_do_search */

void
search_button_clicked(GtkWidget *button)
{
    GtkWidget *entry;

    entry = GET_WIDGET("search_entry");
    rc_gui_do_search(gtk_entry_get_text(GTK_ENTRY(entry)));
} /* search_button_clicked */

void
search_box_activated(GtkWidget *entry)
{
    rc_gui_do_search(gtk_entry_get_text(GTK_ENTRY(entry)));
} /* search_box_activated */

/* CB */
static void
on_set_base (GtkHTML *html, const gchar *url, gpointer data)
{
    char *url2;

    url2 = gtk_object_get_data (GTK_OBJECT (html), "base_url");
    g_free (url2);

    rc_debug (RC_DEBUG_LEVEL_INFO, "base url: %s\n", url);
    gtk_object_set_data (GTK_OBJECT (html), "base_url",
                         g_strdup (url));
}

/* CB */
void
dont_show_clicked(GtkToggleButton *widget)
{
    gboolean selected = gtk_toggle_button_get_active(widget);

    gui_prefs_set_signature_ask(!selected);
} /* dont_show_clicked */

/* CB */
void
select_all_update(GtkWidget *item)
{
    select_all_updates(appinfo->current_channel);
} /* select_all_update */

/* CB */
void
select_none_update(GtkWidget *item)
{
    unselect_all_updates(appinfo->current_channel);
} /* select_none_update */

/* CB */
void
select_all_install(GtkWidget *item)
{
    select_all_available(appinfo->current_channel);
} /* select_all_install */

/* CB */
void
select_none_install(GtkWidget *item)
{
    unselect_all_available(appinfo->current_channel);
} /* select_none_install */

/* CB */
void
select_none_remove(GtkWidget *item)
{
    unselect_all_installed(appinfo->current_channel);
} /* select_none_remove */

/* CB */
void
on_install_local_file_activate(GtkWidget *item)
{
    rc_gui_install_local_file();
}

/* CB */
void
on_verify_installed_packages_activate(GtkWidget *item)
{
    rc_gui_verify_installed_packages();
}
