/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-log-display.c: Display windows for the download/install/remove logs.
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include "gui.h"
#include "gui-log-display.h"

GtkWidget *windows[LOG_LAST] = { NULL };

static void
log_appended(GUILog *log, const char *message, GtkCList *clist)
{
    char *text[] = { (char *) message };

    gtk_clist_append(clist, text);
} /* log_appended */

static void
log_reset(GUILog *log, GtkCList *clist)
{
    gtk_clist_clear(clist);
} /* log_reset */

static gboolean
window_deleted(GtkWidget *w, GdkEvent *event, gpointer data)
{
    LogIndex index = GPOINTER_TO_INT(data);

    windows[index] = NULL;

    return FALSE;
} /* window_deleted */

static void
window_destroyed(GtkWidget *w, gpointer data)
{
    LogIndex index = GPOINTER_TO_INT(data);
    GtkWidget *clist;

    windows[index] = NULL;

    clist = gtk_object_get_user_data(GTK_OBJECT(w));
    gtk_signal_disconnect_by_func(
        GTK_OBJECT(appinfo->log[index]),
        GTK_SIGNAL_FUNC(log_appended), clist);
    gtk_signal_disconnect_by_func(
        GTK_OBJECT(appinfo->log[index]),
        GTK_SIGNAL_FUNC(log_reset), clist);
} /* window_destroyed */

GtkWidget *
rc_gui_log_display(LogIndex index)
{
    GtkWidget *w;

    if (windows[index])
        w = windows[index];
    else {
        char *titles[] = {
            "Download Log",
            "Verification Log",
            "Installation Log",
            "Configuration Log",
            "Removal Log",
            NULL
        };
        char *t;
        GtkWidget *sw;
        GtkWidget *clist;
        GSList *log;
        GSList *i;

        w = windows[index] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_wmclass(GTK_WINDOW(w), "redcarpet foo", "redcarpet bar");
        gtk_window_set_default_size(GTK_WINDOW(w), 600, 200);
        t = g_strdup_printf("Red Carpet - %s", titles[index]);
        gtk_window_set_title(GTK_WINDOW(w), t);
        g_free(t);

        clist = gtk_clist_new(1);
        gtk_clist_column_titles_hide(GTK_CLIST(clist));
        gtk_object_set_user_data(GTK_OBJECT(w), clist);

        sw = gtk_scrolled_window_new(
            gtk_clist_get_hadjustment(GTK_CLIST(clist)),
            gtk_clist_get_vadjustment(GTK_CLIST(clist)));
        gtk_scrolled_window_set_policy(
            GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

        gtk_container_add(GTK_CONTAINER(w), sw);
        gtk_container_add(GTK_CONTAINER(sw), clist);

        log = gui_log_get(appinfo->log[index]);
        i = log;
        while (i) {
            char *message = (char *) i->data;
            char *text[] = { message };

            gtk_clist_append(GTK_CLIST(clist), text);

            i = i->next;
        }

        gtk_signal_connect(
            GTK_OBJECT(appinfo->log[index]), "appended",
            GTK_SIGNAL_FUNC(log_appended), clist);
        gtk_signal_connect(
            GTK_OBJECT(appinfo->log[index]), "reset",
            GTK_SIGNAL_FUNC(log_reset), clist);
        gtk_signal_connect(
            GTK_OBJECT(w), "delete_event",
            GTK_SIGNAL_FUNC(window_deleted), GINT_TO_POINTER(index));
        gtk_signal_connect(
            GTK_OBJECT(w), "destroy",
            GTK_SIGNAL_FUNC(window_destroyed), GINT_TO_POINTER(index));

        gtk_widget_show(sw);
        gtk_widget_show(clist);
    }

    gtk_widget_show(w);

    return w;
} /* rc_gui_log_display */
        
void
rc_gui_log_display_reset_all(void)
{
    int i;

    for (i = 0; i < LOG_LAST; i++)
        gui_log_reset(appinfo->log[i]);
} /* rc_gui_log_display_reset_all */
