/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui-register.c: User registration
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <fcntl.h>
#include <crypt.h>
#include <time.h>

#include <gnome.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include <libgnomeui/gnome-window-icon.h>

#include <gtkhtml/htmlimage.h>
#include <gtkhtml/htmlengine-search.h>

#include "gui.h"
#include "gui-prefs.h"
#include "gui-util.h"
#include "cache.h"
#include "gui-install.h"
#include "mirror.h"
#include "subscription.h"

#include "libredcarpet.h"

#include "gui-register.h"

typedef struct {
    char *widget_name;
    enum {
        TYPE_ENTRY,
        TYPE_OPTION_MENU,
        TYPE_CHECK_BOXES
    } type;
    gpointer data;
    char *html_text;
    GtkWidget *widget;
} WidgetInfo;

char *soft_menu[] = {
    "Productivity Software",
    "Games",
    "Accounting and Finance",
    "Email or Groupware",
    "Development tools",
    "Graphics",
    "Other",
    NULL
};

char *spend_menu[] = {
    "$0 - $100",
    "$100 - $500",
    "$500 - $1000",
    "$1000 - $5000",
    "$5000 - $10000",
    "$10000+",
    NULL
};

char *computers_menu[] = {
    "1-5 at home",
    "6-10 at home",
    "more than 10 at home",
    "1-5 at work",
    "6-10 at work",
    "more than 10 at work",
    NULL
};

char *conn_menu[] = {
    "28.8k modem or slower",
    "56k modem",
    "ISDN",
    "Cable modem",
    "DSL",
    "T1 or faster",
    NULL
};

static WidgetInfo widgetinfo[] = {
    { "question1", TYPE_CHECK_BOXES, soft_menu,
      "What types of Linux software are you currently running?" },
    { "question2", TYPE_CHECK_BOXES, soft_menu,
      "What types of Linux software are you currently looking for?" },
    { "question3", TYPE_OPTION_MENU, spend_menu,
      "How much did you spend in the last year (in US dollars) on software?" },
    { "question4", TYPE_CHECK_BOXES, computers_menu,
      "How many computers are you responsible for?" },
    { "question5", TYPE_OPTION_MENU, conn_menu,
      "What kind of connection to the Internet do you have?" },
    { "question6", TYPE_ENTRY, NULL,
      "In what country do you live?" },
    { NULL }
};

gboolean
rc_gui_registered(void)
{
    gboolean registered;

    gnome_config_push_prefix("/red-carpet/");
    registered = gnome_config_get_bool("Registration/Registered");
    gnome_config_pop_prefix();

    return registered;
} /* rc_registered */

static void
apply_registration(GtkButton *button, GtkHTML *html)
{
    GArray *array;
    char *text;
    WidgetInfo *wi;
    FILE *f;
    char *sendmail;
    int i;
    GtkWidget *w;

    array = g_array_new(TRUE, TRUE, sizeof(char *));
    for (wi = widgetinfo; wi->widget_name; wi++) {
        switch (wi->type) {
        case TYPE_ENTRY:
            text = gtk_entry_get_text(GTK_ENTRY(wi->widget));
            if (!*text || !*text == ' ')
                goto showmessage;
            g_array_append_val(array, text);
            break;
        case TYPE_OPTION_MENU: {
            GtkWidget *menu;
            GtkWidget *item;

            menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(wi->widget));
            item = gtk_menu_get_active(GTK_MENU(menu));
            if (gtk_object_get_data(GTK_OBJECT(item), "notvalid"))
                goto showmessage;
            text = gtk_object_get_data(GTK_OBJECT(item), "text");
            g_array_append_val(array, text);
            break;
        }
        case TYPE_CHECK_BOXES: {
            GSList *l;
            int i = 0;
            char **text;
            char *t;

            l = gtk_object_get_data(GTK_OBJECT(wi->widget), "widget_list");
            text = g_new0(char *, g_slist_length(l) + 1);
            while (l) {
                GtkWidget *w = l->data;

                if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
                    text[i] = gtk_object_get_data(GTK_OBJECT(w), "text");
                    i++;
                }

                l = l->next;
            }
            
            t = g_strjoinv("; ", text);
            g_free(text);
            g_array_append_val(array, t);
            break;
        }
        }
    }
            
    if (g_file_exists("/usr/sbin/sendmail"))
        sendmail = "/usr/sbin/sendmail -i -t";
    else
        sendmail = "/usr/lib/sendmail -i -t";

    f = popen(sendmail, "w");
    if (!f) {
        rc_debug(
            RC_DEBUG_LEVEL_WARNING, "Unable to fork sendmail process.\n");
        gtk_main_quit();
        return;
    }

    fprintf(f, "To: register@ximian.com\n\n");
    for (i = 0; g_array_index(array, char *, i); i++) {
        fprintf(
            f, "%s: %s\n", widgetinfo[i].widget_name, 
            g_array_index(array, char *, i));
    }
    fflush(f);
    pclose(f);

    gnome_config_push_prefix("/red-carpet/");
    gnome_config_set_bool("Registration/Registered", TRUE);
    gnome_config_pop_prefix();
    gnome_config_sync();

    gtk_main_quit();
    g_array_free(array, FALSE);
    return;

showmessage:
    g_array_free(array, FALSE);
    w = gnome_message_box_new(
        "All fields must be filled in.", 
        GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_OK, NULL);
    gtk_widget_show(w);
    gnome_dialog_run_and_close(GNOME_DIALOG(w));
} /* apply_registration */

void
rc_gui_register(void)
{
    GtkWidget *frame;
    GtkWidget *html;
    GtkHTMLStream *stream;
    char *h;
    WidgetInfo *wi;
    int i;

    rc_gui_set_page("main_notebook", PAGE_REGISTER);

    frame = GET_WIDGET("register_html");
    g_assert(frame);
    html = gtk_object_get_user_data(GTK_OBJECT(frame));
    g_assert(html);
    
    /* Desensitize inappropriate widgets */
    rc_gui_set_refresh_item(REFRESH_SUMMARY);
    gtk_widget_set_sensitive(GET_WIDGET("summary_refresh"), FALSE);
    gtk_widget_set_sensitive(GET_WIDGET("subscribe1"), FALSE);
    gtk_widget_set_sensitive(GET_WIDGET("unsubscribe1"), FALSE);

    for (wi = widgetinfo; wi->widget_name; wi++) {
        switch (wi->type) {
        case TYPE_ENTRY:
            wi->widget = gtk_entry_new();
            break;
        case TYPE_OPTION_MENU: {
            GtkWidget *menu, *item;
            int j;
            char **label = (char **) wi->data;

            menu = gtk_menu_new();
            item = gtk_menu_item_new_with_label("(select one)");
            gtk_object_set_data(
                GTK_OBJECT(item), "notvalid", GINT_TO_POINTER(TRUE));
            gtk_menu_append(GTK_MENU(menu), item);
            for (j = 0; label[j]; j++) {
                item = gtk_menu_item_new_with_label(label[j]);
                gtk_object_set_data(GTK_OBJECT(item), "text", label[j]);
                gtk_menu_append(GTK_MENU(menu), item);
            }
            wi->widget = gtk_option_menu_new();
            gtk_option_menu_set_menu(GTK_OPTION_MENU(wi->widget), menu);
            break;
        }
        case TYPE_CHECK_BOXES: {
            GtkWidget *w;
            int j;
            gboolean first = TRUE;
            char **label = (char **) wi->data;
            GSList *list = NULL;

            for (j = 0; label[j]; j++);
            wi->widget = gtk_table_new(j / 2 + j % 2, 2, TRUE);

            for (j = 0; label[j]; j++) {
                w = gtk_check_button_new_with_label(label[j]);
                gtk_table_attach_defaults(
                    GTK_TABLE(wi->widget), w, first ? 0 : 1, first ? 1 : 2,
                    j / 2, j / 2 + 1);
                gtk_object_set_data(GTK_OBJECT(w), "text", label[j]);
                list = g_slist_append(list, w);
                first = !first;
            }
            gtk_object_set_data(GTK_OBJECT(wi->widget), "widget_list", list);
            break;
        }
        default:
            g_assert_not_reached();
            break;
        }
        gtk_object_set_data(GTK_OBJECT(html), wi->widget_name, wi->widget);
        gtk_widget_set_usize(wi->widget, 300, -1);
        gtk_widget_show_all(wi->widget);
    }

    gtk_signal_connect(
        GTK_OBJECT(GET_WIDGET("register_ok")), "clicked",
        GTK_SIGNAL_FUNC(apply_registration), html);
    
    stream = gtk_html_begin(GTK_HTML(html));
    h = g_strdup_printf(
        "<html><body bgcolor=\"#e3e5e5\" text=\"#000000\" notlink=\"#e5e3e3\""
        "            marginwidth=0 marginheight=0>"
        "<table cellspacing=0 cellbadding=1 width=\"100%%\" border=0>"
        "<tr><td bgcolor=\"#000000\" width=36 align=left>"
        "<img src=\"file://"SHAREDIR"/key.png\"></td>"
        "<td bgcolor=\"#000000\" align=left width=\"100%%\">"
        "<font size=\"+2\" color=\"#ffffff\">"
        "<b>Welcome to Red Carpet</b></font></td></tr>"
        "<tr><td colspan=2 height=\"100%%\">"
        "<center><table cellspacing=2 cellpadding=2 width=\"90%%\" border=0>"
        "<tr><td>"
        "<p>Before we get started, please take a few moments to register Red "
        "Carpet. Registering your software will help us serve you better. "
        "Rest assured that we will do a good job protecting your privacy. Our "
        "privacy policy can be found online at "
        "<a href=\"www.ximian.com/privacy\">http://www.ximian.com/privacy</a>."
        "</p></td></tr></table>"
        "<table cellspacing=2 cellpadding=2 width=95%% border=0>");
    gtk_html_write(GTK_HTML(html), stream, h, strlen(h));
    g_free(h);

    for (wi = widgetinfo, i = 1; wi->widget_name; wi++, i++) {
        h = g_strdup_printf(
            "<tr><td align=center colspan=2><font size=-1><b>%d. %s"
            "</b></font><br><br>"
            "<object classid=\"%s\"></object><br><br><br></td></tr>",
            i, wi->html_text, wi->widget_name);
        gtk_html_write(GTK_HTML(html), stream, h, strlen(h));
        g_free(h);
    }
    
    h = "</table></center></td></tr></table></body></html>";
    gtk_html_write(GTK_HTML(html), stream, h, strlen(h));
    gtk_widget_show_now(GET_WIDGET("main_window"));
    gtk_widget_draw(GET_WIDGET("main_window"), NULL);
    gtk_main();
    gtk_widget_hide(GET_WIDGET("main_window"));
    rc_gui_set_page("main_notebook", PAGE_CHANNELS);
} /* rc_gui_register */
