/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-transfer.h: Wrapper around GnomeTransfer that does blocking transfers
 * and handles caching
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GUI_TRANSFER_H
#define _GUI_TRANSFER_H

#include <gtk/gtk.h>
#include "gnome-transfer.h"

#define GUI_TYPE_TRANSFER                (gui_transfer_get_type())
#define GUI_TRANSFER(obj)                (GTK_CHECK_CAST((obj), \
                                          GUI_TYPE_TRANSFER, GUITransfer))
#define GUI_TRANSFER_CLASS(klass)        (GTK_CHECK_CLASS_CAST((klass), \
                                          GUI_TYPE_TRANSFER, GUITransferClass))
#define GUI_IS_TRANSFER(obj)             (GTK_CHECK_TYPE((obj), \
                                          GUI_TYPE_TRANSFER))
#define GUI_IS_TRANSFER_CLASS(klass)     (GTK_CHECK_CLASS_TYPE((klass), \
                                          GUI_TYPE_TRANSFER))
#define GUI_TRANSFER_GET_CLASS(obj)      (GUI_TRANSFER_CLASS( \
                                          GTK_OBJECT(obj)->klass))
typedef   enum _GUITransferError GUITransferError;
typedef struct _GUITransfer      GUITransfer;
typedef struct _GUITransferClass GUITransferClass;

enum _GUITransferError {
    GUI_TRANSFER_ERROR_NONE,
    GUI_TRANSFER_ERROR_RETRY,
    GUI_TRANSFER_ERROR_ABORT
};

struct _GUITransfer {
    GtkObject parent;

    GnomeTransfer *gt;

    GUITransferError error;

    char *url;
    char *filename;
    GByteArray *data;

    gboolean nocache;

    gboolean block;
    gboolean error_notify;

    gboolean aborted;
    
    int file_size;
    int trans_size;

    GtkWidget *progress_bar;
    GtkWidget *label;

    gboolean paused;
    int offset;
    GTimer *timer;
};

struct _GUITransferClass {
    GtkObjectClass parent_class;

    /* Signals */

    void (*file_data)(GUITransfer *, char *buffer, int size);
    void (*file_done)(GUITransfer *);
};

guint gui_transfer_get_type(void);
GUITransfer *gui_transfer_new(gboolean blocking, gboolean error_notify);
GByteArray *gui_transfer(GUITransfer *t, gchar *url);
void gui_transfer_set_proxy_info(GUITransfer *t, const char *url, 
                                 const char *name, const char *pass);
void gui_transfer_set_progress_bar(GUITransfer *t, GtkProgress *progress_bar);
void gui_transfer_set_progress_label(GUITransfer *guit, GtkLabel *label);
GSList *gui_transfer_get_current_transfers(void);
GUITransferError gui_transfer_get_error(GUITransfer *t);
void gui_transfer_set_cache(GUITransfer *t, gboolean enabled);

#endif /* _GUI_TRANSFER_H */
