/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui-uri.c: URI Handler
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <fcntl.h>
#include <crypt.h>
#include <time.h>

#include <gnome.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include <libgnomeui/gnome-window-icon.h>

#include <gtkhtml/htmlengine-search.h>
#include <gtkhtml/htmlimage.h>

#include "gui.h"
#include "gui-log-display.h"
#include "gui-transfer.h"
#include "gui-prefs.h"
#include "cache.h"
#include "gui-install.h"
#include "mirror.h"
#include "subscription.h"

#include "libredcarpet.h"

#include "gui-uri.h"
#include "gui-util.h"
#include "gui-html.h"
#include "gui-glade.h"
#include "gui-channel.h"

#include "subscription.h"

static void
action_button_sensitive (gboolean sensitive)
{
    GtkWidget *action_button;

    action_button = (GtkWidget *) gtk_object_get_data (GTK_OBJECT(appinfo->action_html),
                                                       "action_button");

    gtk_widget_set_sensitive (action_button, sensitive ? TRUE : FALSE);
}

/* CB */
void
rc_gui_html_link_clicked(GtkHTML *html, const char *uri, GUIChannel *channel)
{
    if (g_strncasecmp(uri, "updater", 7) == 0)
        rc_gui_handle_uri(html, uri, channel);
    else
        gnome_url_show(uri);
} /* rc_gui_html_link_clicked */

/* CB */
void
rc_gui_html_load_image(GtkHTML *html, char *url, GtkHTMLStream *stream)
{
    char *das_url = NULL;
    char *base_url;

    g_return_if_fail(html);

    if (g_strncasecmp(url, "updater", 6) == 0) {
        GUIChannel *c;

        if (strncmp (url + strlen ("updater:"), "check", 5) == 0) {
            /* it's a check on/off */
            if (strcmp (url + strlen ("updater:check_"), "inst") == 0) {
                das_url = g_strdup (CKINSTALL_IMAGE_URL);
            } else if (strcmp (url + strlen ("updater:check_"), "rem") == 0) {
                das_url = g_strdup (CKREMOVE_IMAGE_URL);
            } else {
                das_url = g_strdup (CKNOTHING_IMAGE_URL);
            }
        } else {
            c = gtk_object_get_data(GTK_OBJECT(html), "channel");

            gtk_object_set_data(GTK_OBJECT(html), "stream", stream);
            rc_gui_handle_uri(html, url, c);
            return;
        }
    }

    if (!das_url) {
        if (!rc_gui_url_is_qualified(url) && url[0] != '/') {
            char *p;

            base_url = gtk_object_get_data (GTK_OBJECT (html),
                                            "base_url");
            /* We're not doing mirroring on these URLs. I think that's okay, as
               I think this will only happen in the info pages. Not sure. */

            /* Handle relative URLs for imagitas */
            if (base_url) {
                p = strrchr (base_url, '/');
                if (p && *(p+1)) {
                    /* URL doesn't end in /, we need to strip the last element */
                    gchar *tmpbase;
                    tmpbase = g_strndup (base_url, p - base_url);
                    das_url = g_strdup_printf ("%s/%s", tmpbase, url);
                    g_free (tmpbase);
                }
            }

            if (!das_url) {
                das_url = (base_url) 
                    ? g_strdup_printf ("%s/%s", base_url, url)
                    : g_strdup (url);
            }
        } else {
            das_url = g_strdup (url);
        }
    }

    rc_debug (RC_DEBUG_LEVEL_INFO, "html_load_image: grabbing %s\n", das_url);

    rc_gui_html_download (html, stream, das_url);

    g_free (das_url);
} /* rc_gui_html_load_image */

/* Note! html might be NULL in some cases (i.e. the buttons at the bottom of each page clicked */
void
rc_gui_handle_uri (GtkHTML *html, const char *uri, GUIChannel *channel)
{
    char **compound;
    char *token;

    g_return_if_fail(uri);

    rc_debug (RC_DEBUG_LEVEL_INFO, "Updater URI clicked: %s\n", uri);

    if (!channel)
        channel = appinfo->current_channel;

    compound = g_strsplit(uri, "#", 1);

    token = strchr(compound[0], ':');
    if (!token) {
        rc_debug (RC_DEBUG_LEVEL_WARNING, "Invalid URI\n");
        g_strfreev(compound);
        return;
    }
    token++;

    /* Note that these have to be in lowercase; we can't downcase, since there
     * might be bits after the main URI that we want to preserve case in.
     */

    if (strcmp(token, "update_page") == 0) {
        rc_gui_set_page("notebook", PAGE_UPDATES);
    }
    else if (strcmp(token, "installed_page") == 0) {
        rc_gui_render_installed_html(channel);
        rc_gui_set_page("notebook", PAGE_INSTALLED);
    }
    else if (strcmp(token, "available_page") == 0) {
        rc_gui_render_available_html(channel);
        rc_gui_set_page("notebook", PAGE_AVAILABLE);
    }
    else if (strcmp(token, "preferences_page") == 0) {
        rc_gui_set_page("notebook", PAGE_PREFERENCES);
    }
    else if (strcmp(token, "info_page") == 0) {
        rc_gui_render_info_html(channel);
        rc_gui_set_page("notebook", PAGE_INFO);
    }
    else if (strcmp(token, "subscribe") == 0) {
        g_return_if_fail(channel);

        rc_gui_channel_subscribe(channel);
    }
    else if (strcmp(token, "unsubscribe") == 0) {
        g_return_if_fail(channel);

        rc_gui_channel_unsubscribe(channel);
    }
    else if (strcmp(token, "summary") == 0) {
        rc_gui_channel_select(NULL);
        rc_gui_show_executive_summary();
    }
    else if (strcmp(token, "news_page") == 0) {
        rc_gui_channel_select(NULL);
        rc_gui_show_news_page();
    }
    else if (strcmp(token, "featured_page") == 0) {
        rc_gui_channel_select(NULL);
        rc_gui_show_featured_page();
    }
    else if (strcmp(token, "subscribed_page") == 0) {
        rc_gui_channel_select(NULL);
        rc_gui_show_channel_page (TRUE);
    }
    else if (strcmp(token, "unsubscribed_page") == 0) {
        rc_gui_channel_select(NULL);
        rc_gui_show_channel_page (FALSE);
    }
    else if (strncmp(token, "channel_name", 12) == 0) {
        char *s;
        GUIChannel *gc;

        s = strchr(token, '?');
        if (!(s && *++s))
            goto done;

        gc = GUICHANNEL(rc_channel_get_by_name(appinfo->channel_list, s));
        if (!gc) {
            rc_debug (RC_DEBUG_LEVEL_ERROR, "gc == NULL (can't find channel %s)\n", s);
            return;
        }

        rc_gui_channel_select(gc);

        channel = gc;
    }
    else if (strncmp(token, "channel", 7) == 0) {
        char *s;
        int cid;
        GUIChannel *gc;

        s = strchr (token, '?');
        if (!(s && *++s)) 
            goto done;

        cid = atoi(s);
        gc = GUICHANNEL(rc_channel_get_by_id(appinfo->channel_list, cid));
        g_return_if_fail(gc);

        rc_gui_channel_select(gc);

        channel = gc;
    }
    else if (strcmp(token, "update") == 0) {
        RCPackageSList **packages = gtk_object_get_data(
            GTK_OBJECT(appinfo->update_html), "packages_to_install");

        g_return_if_fail(channel);

        if (!packages || !*packages)
            goto done;

        gui_install_packages(channel, *packages);
    }
    else if (strcmp(token, "install") == 0) {
        RCPackageSList **packages = gtk_object_get_data(
            GTK_OBJECT(appinfo->available_html), "packages_to_install");
        
        g_return_if_fail(channel);

        if (!packages || !*packages)
            goto done;

        gui_install_packages(channel, *packages);
    }
    else if (strcmp(token, "remove") == 0) {
        RCPackageSList **packages = gtk_object_get_data(
            GTK_OBJECT(appinfo->installed_html), "packages_to_remove");

        if (!packages || !*packages)
            goto done;

        gui_remove_packages(channel, *packages);
    }
    else if (strcmp(token, "install_all") == 0) {
        RCPackageSList *uberlist = NULL;
        GSList *i;

        i = appinfo->channel_list;
        while (i) {
            GUIChannel *c = i->data;

            if (c->subscribed) {
                uberlist = g_slist_concat(
                    uberlist, g_slist_copy (c->available_updates));
            }

            i = i->next;
        }

        gui_install_packages(NULL, uberlist);
        g_slist_free(uberlist);
    }
    else if (strcmp(token, "prefs_proxy") == 0) {
#if 0
        GtkWidget *notebook;
        notebook = GET_WIDGET("prefs_notebook");
        g_assert(notebook);
#endif

        on_preferences1_activate(NULL, NULL);
#if 0
        gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
#endif
    }
    else if (strncmp(token, "info", 4) == 0) {
        char *s;
        RCPackage *package;

        s = strchr(token, '?');
        if (!(s && *++s)) goto done;
        package = (RCPackage *) strtoul(s, NULL, 16);
        rc_gui_render_package_info_html(package);
    }
    else if (strcmp(token, "close") == 0) {
        gtk_widget_hide(GET_WIDGET("package_vbox"));
    }
    else if (strncmp(token, "install_toggle", 14) == 0) {
        char *s, *p;
        char *imgname;
        RCPackage *pkg;
        RCPackageSList **packages;
        HTMLImage *hi;
        unsigned long a;

        s = strchr (token, '?');
        if (!(s && *++s)) 
            goto done;

        a = strtoul (s, &p, 0);
        imgname = g_strdup_printf ("ckimg%ld", a);

        if (!(p && *++p)) {
            g_free (imgname);
            goto done;
        }

        /* *** FIXME *** this is not 64-bit safe */
        /* this is a pointer address, so... :-P */
        pkg = (RCPackage *) strtoul (p, NULL, 16);
        packages = gtk_object_get_data(
            GTK_OBJECT(html), "packages_to_install");
        hi = gtk_html_get_object_by_id (html, imgname);
        if (!hi) {
            rc_debug(
                RC_DEBUG_LEVEL_WARNING, 
                "Failed in getting htmlimage for %s\n", imgname);
        }

        if (g_slist_find (*packages, pkg)) {
            *packages = g_slist_remove (*packages, pkg);
            if (hi) 
                html_image_set_url(hi, "updater:check_nothing");
        } 
        else {
            *packages = g_slist_append (*packages, pkg);
            if (hi) 
                html_image_set_url(hi, "updater:check_inst");
        }

        action_button_sensitive (g_slist_length (*packages));

        gtk_widget_queue_draw (GTK_WIDGET (html));
        g_free (imgname);
    }
    else if (strncmp(token, "remove_toggle", 13) == 0) {
        char *s, *p;
        char *imgname;
        RCPackage *pkg;
        RCPackageSList **packages;
        HTMLImage *hi;
        unsigned long a;

        s = strchr (token, '?');
        if (!(s && *++s)) 
            goto done;

        a = strtoul (s, &p, 0);
        imgname = g_strdup_printf ("ckimg%ld", a);

        if (!(p && *++p)) {
            g_free (imgname);
            goto done;
        }

        /* *** FIXME *** this is not 64-bit safe */
        /* this is a pointer address, so... :-P */
        pkg = (RCPackage *) strtoul (p, NULL, 16);

        packages = gtk_object_get_data (GTK_OBJECT(html), "packages_to_remove");
        hi = gtk_html_get_object_by_id (html, imgname);
        if (!hi) {
            rc_debug (RC_DEBUG_LEVEL_WARNING, "Failed in getting htmlimage for %s\n", imgname);
        }
        if (g_slist_find (*packages, pkg)) {
            *packages = g_slist_remove (*packages, pkg);
            if (hi) 
                html_image_set_url (hi, "updater:check_nothing");
        } 
        else {
            *packages = g_slist_append (*packages, pkg);
            if (hi) 
                html_image_set_url (hi, "updater:check_rem");
        }

        action_button_sensitive (g_slist_length (*packages));

        gtk_widget_queue_draw (GTK_WIDGET (html));
        g_free (imgname);
    }
    else if (strncmp(token, "install_set", strlen("install_set")) == 0) {
#ifdef GUI_USE_PACKAGESETS
        gchar *sname;
        sname = strchr (token, '?');
        if (p) {
            GSList *l;
            RCPackageSet *ps;
            RCPackageSList **plist;
            int i;

            sname++; /* skip '?' */

            /* Find the package set */
            l = channel->rcchannel.package_sets;
            while (l) {
                ps = (RCPackageSet *) l->data;
                if (strcmp (sname, ps->name) == 0) {
                    /* This one */
                    break;
                }
                l = l->next;
            }

            if (!l) {
                rc_debug (RC_DEBUG_LEVEL_WARNING,
                    "got install_set for non-valid package set: '%s'\n", sname);
                goto done;
            }

            plist = gtk_object_get_data(
                GTK_OBJECT(appinfo->available_html), "packages_to_install");
            if (plist && *plist) {
                g_slist_free(*plist);
                *plist = NULL;
            } else if (!plist) {
                plist = g_new0(RCPackageSList *, 1);
                *plist = NULL;
                gtk_object_set_data(
                    GTK_OBJECT(appinfo->available_html), "packages_to_install",
                    plist);
            }

            /* ps now has this package set, we need to add these packages
             * and simulate a install */
            l = ps->packages;
            while (l) {
                RCPackage *pkg;
                gchar *pname = (gchar *) l->data;
                /* We need to find this package */
                pkg = pkginfo_find_package(
                    &channel->rcchannel, pname, channel->subchannel_preference);
                if (!pkg) {
                    rc_debug (RC_DEBUG_LEVEL_WARNING, 
                        "Can't find package named '%s' in set '%s' in channel "
                        "id %d\n", pname, ps->name, channel->rcchannel.id);
                    l = l->next;
                    continue;
                }

                *plist = g_slist_append (*plist, pkg);
                l = l->next;
            }

            /* pretend we just hit install */
            rc_gui_handle_uri (html, "updater:install", channel);
        }
#endif
    }
    else if (strcmp(token, "select_all") == 0)
        select_all_updates(channel);
    else if (strcmp(token, "unselect_all") == 0)
        unselect_all_updates(channel);
    else if (strcmp(token, "select_all_available") == 0)
        select_all_available(channel);
    else if (strcmp(token, "unselect_all_available") == 0)
        unselect_all_available(channel);
    else if (strcmp(token, "unselect_all_installed") == 0)
        unselect_all_installed(channel);
    else if (strcmp(token, "download_log") == 0)
        rc_gui_log_display(LOG_DOWNLOAD);
    else if (strcmp(token, "verify_log") == 0)
        rc_gui_log_display(LOG_VERIFY);
    else if (strcmp(token, "install_log") == 0)
        rc_gui_log_display(LOG_INSTALL);
    else if (strcmp(token, "config_log") == 0)
        rc_gui_log_display(LOG_CONFIG);
    else if (strcmp(token, "remove_log") == 0)
        rc_gui_log_display(LOG_REMOVE);
    else if (strncmp(token, "search", 6) == 0) {
        char *s;

        s = strchr (token, '?');
        if (!(s && *++s)) 
            goto done;
        rc_gui_do_search(s);
    }
    else if (strncmp(token, "web", 3) == 0) {
        char *s;
        char *h;
        GtkHTMLStream *stream;

        s = strchr(token, '?');
        if (!(s && *++s))
            goto done;

        stream = gtk_html_begin(html);
        h = "<html><body><b>Please wait, loading page...</b></body></html>";
        gtk_html_write(html, stream, h, strlen(h));
        gtk_html_end(html, stream, GTK_HTML_STREAM_OK);
        rc_gui_html_download(html, NULL, s);

        /* FIXME: Valid HTTP URLs can contain octothorpes, as they indicate
           anchors. Instead of doing the right thing and escaping them, we're
           just going to prohibit compound URIs beyond the web URI. This'll
           get fixed if there is ever a compelling reason for them. */
        g_strfreev(compound);
        return;
    }
    else if (strcmp(token, "noop") == 0) {
        /* No-op */
    }

  done:
    /* If there is a compound URI, we want to recurse. */
    if (compound[1] && html)
        rc_gui_handle_uri(html, compound[1], channel);
        
    g_strfreev(compound);
} /* rc_gui_handle_uri */

void
select_all_updates(GUIChannel *channel)
{
    int count, count_max;
    char *key;
    HTMLImage *hi;
    RCPackageSList **packages;

    count_max = GPOINTER_TO_INT(
        gtk_object_get_data(GTK_OBJECT(appinfo->update_html), "checkbox_count"));
    key = g_malloc(100);       /* the biggest int will fit in this */
    for (count = 0; count < count_max; count++) {
        sprintf(key, "ckimg%d", count);
        hi = gtk_html_get_object_by_id(appinfo->update_html, key);
        html_image_set_url(hi, "updater:check_inst");
    }
    g_free (key);

    packages = gtk_object_get_data(
        GTK_OBJECT(appinfo->update_html), "packages_to_install");
    g_slist_free(*packages);
    *packages = g_slist_copy(channel->available_updates);

    action_button_sensitive (g_slist_length (*packages));

    /* And then we render */
    gtk_widget_queue_draw(GTK_WIDGET(appinfo->update_html));
} /* select_all_updates */

void
unselect_all_updates(GUIChannel *channel)
{
    int count, count_max;
    char *key;
    HTMLImage *hi;
    RCPackageSList **packages;

    count_max = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(appinfo->update_html), "checkbox_count"));
    key = g_malloc (100);       /* the biggest int will fit in this */
    for (count = 0; count < count_max; count++) {
        sprintf (key, "ckimg%d", count);
        hi = gtk_html_get_object_by_id (appinfo->update_html, key);
        html_image_set_url (hi, "updater:check_nothing");
    }
    g_free (key);

    packages = gtk_object_get_data (GTK_OBJECT (appinfo->update_html), "packages_to_install");
    g_slist_free (*packages);
    *packages = NULL;

    action_button_sensitive (g_slist_length (*packages));

    /* And then we render */
    gtk_widget_queue_draw (GTK_WIDGET (appinfo->update_html));
} /* unselect_all_updates */

void
select_all_available(GUIChannel *channel)
{
    int count, count_max;
    char *key;
    HTMLImage *hi;
    RCPackageSList **packages;

    count_max = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(appinfo->available_html), "checkbox_count"));
    key = g_malloc (100);       /* the biggest int will fit in this */
    for (count = 0; count < count_max; count++) {
        sprintf (key, "ckimg%d", count);
        hi = gtk_html_get_object_by_id (appinfo->available_html, key);
        html_image_set_url (hi, "updater:check_inst");
    }
    g_free (key);

    packages = gtk_object_get_data (GTK_OBJECT (appinfo->available_html), "packages_to_install");
    g_slist_free (*packages);
    *packages = g_slist_copy (channel->uninstalled_packages);

    action_button_sensitive (g_slist_length (*packages));

    /* And then we render */
    gtk_widget_queue_draw (GTK_WIDGET (appinfo->available_html));
} /* select_all_available */

void
unselect_all_available(GUIChannel *channel)
{
    int count, count_max;
    char *key;
    HTMLImage *hi;
    RCPackageSList **packages;

    count_max = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(appinfo->available_html), "checkbox_count"));
    key = g_malloc (100);       /* the biggest int will fit in this */
    for (count = 0; count < count_max; count++) {
        sprintf (key, "ckimg%d", count);
        hi = gtk_html_get_object_by_id (appinfo->available_html, key);
        html_image_set_url (hi, "updater:check_nothing");
    }
    g_free (key);

    packages = gtk_object_get_data (GTK_OBJECT (appinfo->available_html), "packages_to_install");
    g_slist_free (*packages);
    *packages = NULL;

    action_button_sensitive (g_slist_length (*packages));

    /* And then we render */
    gtk_widget_queue_draw (GTK_WIDGET (appinfo->available_html));
} /* unselect_all_available */

void
unselect_all_installed(GUIChannel *channel)
{
    int count, count_max;
    char *key;
    HTMLImage *hi;
    RCPackageSList **packages;

    count_max = GPOINTER_TO_INT(gtk_object_get_data(
        GTK_OBJECT(appinfo->installed_html), "checkbox_count"));
    key = g_malloc(100);       /* the biggest int will fit in this */
    for (count = 0; count < count_max; count++) {
        sprintf(key, "ckimg%d", count);
        hi = gtk_html_get_object_by_id(appinfo->installed_html, key);
        html_image_set_url(hi, "updater:check_nothing");
    }
    g_free (key);

    packages = gtk_object_get_data(
        GTK_OBJECT(appinfo->installed_html), "packages_to_remove");
    g_slist_free(*packages);
    *packages = NULL;

    action_button_sensitive (g_slist_length (*packages));

    /* And then we render */
    gtk_widget_queue_draw (GTK_WIDGET (appinfo->installed_html));
} /* unselect_all_installed */

typedef struct {
    GtkHTML *html;
    GtkHTMLStream *stream;
} GUIHTMLandStream;


/* CB */
static void
render_html_cb(GUITransfer *t, char *buf, int size,
               GUIHTMLandStream *ghas)
{
    if (size > 0)
        gtk_html_write(ghas->html, ghas->stream, buf, size);
} /* render_html_cb */

/* CB */
static void
finish_html_cb(GUITransfer *t, GUIHTMLandStream *ghas)
{
    if (t->gt->status == GNOME_TRANSFER_COMPLETE)
        gtk_html_end(ghas->html, ghas->stream, GTK_HTML_STREAM_OK);
    else
        gtk_html_end(ghas->html, ghas->stream, GTK_HTML_STREAM_ERROR);
    
    g_free (ghas);
} /* finish_html_cb */

/* CB */
static void
free_gui_transfer_cb(GUITransfer *t)
{
    GSList *iter = appinfo->channel_transfers;

    while (iter) {
        GSList *next = iter->next;
        GUITransfer *tmp_t = (GUITransfer *)(iter->data);

        if (t == tmp_t) {
            appinfo->channel_transfers = g_slist_remove_link(
                appinfo->channel_transfers, iter);
            g_slist_free_1(iter);
        }

        iter = next;
    }

    gtk_object_destroy(GTK_OBJECT(t));
} /* free_gui_transfer_cb */

void
rc_gui_html_download(GtkHTML *html, GtkHTMLStream *stream_in, gchar *url)
{
    GtkHTMLStream *stream = stream_in;
    char *u;

    g_return_if_fail(html);

    /* This is not an error. */
    if (!url)
        return;

    if (!rc_gui_url_is_qualified(url) &&
        gtk_object_get_data(GTK_OBJECT(html), "base_url")) {
        u = g_strconcat(
            gtk_object_get_data(GTK_OBJECT(html), "base_url"), url, NULL);
    }
    else
        u = g_strdup(url);

    if (html && !stream_in)
        stream = gtk_html_begin (html);

    if (rc_gui_url_is_qualified(u)) {
        GUITransfer *t = gui_transfer_new(FALSE, FALSE);
        GUIHTMLandStream *ghas = g_new0 (GUIHTMLandStream, 1);

        ghas->stream = stream;
        ghas->html = html;

        gtk_signal_connect(
            GTK_OBJECT(t), "file_data",
            GTK_SIGNAL_FUNC(render_html_cb), ghas);

        gtk_signal_connect(
            GTK_OBJECT(t), "file_done",
            GTK_SIGNAL_FUNC(finish_html_cb), ghas);

        gtk_signal_connect_after(
            GTK_OBJECT(t), "file_done",
            GTK_SIGNAL_FUNC(free_gui_transfer_cb), NULL);

        appinfo->channel_transfers =
            g_slist_append(appinfo->channel_transfers, t);

        gui_transfer(t, u);
    } 
    else {
        int fd;
        int bytes;
        char buf[1024];

        fd = open(u, O_RDONLY);
        if (fd < 0) {
            rc_debug(RC_DEBUG_LEVEL_WARNING, "Unable to load image `%s'\n", u);
            gtk_html_end(html, stream, GTK_HTML_STREAM_ERROR);
            g_free(u);
            return;
        }

        do {
            bytes = read(fd, buf, 1024);
            if (bytes > 0) {
                gtk_html_write(html, stream, buf, bytes);
            }
        } while (bytes > 0);

        if (bytes < 0) {
            gtk_html_end(html, stream, GTK_HTML_STREAM_ERROR);
        } else {
            gtk_html_end(html, stream, GTK_HTML_STREAM_OK);
        }
    }

    g_free(u);
} /* rc_gui_html_download */
