/* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * gui-util.h: GUI Utility functions
 *
 * Copyright 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GUI_UTIL_H
#define _GUI_UTIL_H

#include <glib.h>

#include "gui.h"
#include "libredcarpet.h"

typedef enum _RCError RCError;

enum _RCError {
    /* This is a fatal error that requires the application quit. This is
       obviously the most severe */
    RC_ERROR_FATAL,
    /* An error condition that should retry the operation */
    RC_ERROR_RETRY,
    /* An error condition that should abort the current operation but not
       the whole application */
    RC_ERROR_ABORT
};

const char *rc_gui_get_host (void);
void rc_gui_query_system_packages (void);
void rc_gui_set_page (char *notebook, int page);
char *rc_gui_format_size(unsigned int size);
char * rc_gui_format_size_with_nbsp(unsigned int size);
char *rc_gui_number_to_word(int number);
const gchar *rc_gui_section_icon_name (RCPackageSection section);
const char *rc_gui_get_pretty_name(const char *package_name);

gboolean rc_gui_error_box(const char *message, RCError err);
void rc_gui_split_channel_list (GUIChannelSList  *channels,
                                RCChannelSList  **subscribed_channels,
                                RCChannelSList  **unsubscribed_channels);
int rc_gui_split_by_package_category(RCPackageSList *packages, 
                                     RCPackageSList ***split_packages);
int rc_gui_split_by_package_importance(RCPackageSList *available_updates,
                                       RCPackageSList *system_packages,
                                       RCPackageSList ***split_packages,
                                       int *total_size);
gboolean rc_gui_url_is_qualified(const char *url);
void rc_gui_set_refresh_item(RefreshType type);
void rc_gui_set_select_item(ActionType action);
void rc_gui_sensitize_select_items(gboolean sensitize);

void rc_gui_install_local_file (void);
void rc_gui_set_menubar_sensitive (gboolean sense);

#endif /* _GUI_UTIL_H */
